#!flask/bin/python
import six
from flask import Flask, jsonify, abort, request, make_response, url_for, render_template, json
from flask.ext.httpauth import HTTPBasicAuth
from flask.ext.mysql import MySQL
from werkzeug import generate_password_hash, check_password_hash
from flask.ext.cors import CORS
import os
 
#This code up API REST to communicate with other services
#Works as backend of nopaas-chat
#provides JSON that can be get via HTTP methods

#initialize consctructor Flask
app = Flask(__name__, static_url_path="")
CORS(app)
auth = HTTPBasicAuth()

#gets the filename reading metadata.yaml
filename=open("../metadata.yaml", 'r')
first_line=filename.readline()
CHARM_NAME=first_line.split(" ")[1][:-1]

#gets charm_name through file path
file_path=os.getcwd()
CHARM_UNIT_NAME=file_path.split('/')[5][5:]

#method auxiliar that inserts user in database 
def insert_user(name, color, ip, cursor, conn):
	sql_insert="INSERT INTO user (name,color,ip) VALUES ('"+name+"','"+color+"','"+ip+"')"
	cursor.execute(sql_insert)
	conn.commit()


#retrieve mysql configurations in log_mysql.txt file
#Case a MySQL instance is not deployed and connects with charm, probably configurations will be writed in lines 4 to 7
#Else, configurations will be writed in 0 to third line
#this happens by an delay in deploy 
try:
	mysql = MySQL()
	
	info=open('../log_mysql.txt', 'r')
	lines=info.readlines()
	# MySQL configurations
	if (lines[0]=="\n"):
                app.config['MYSQL_DATABASE_USER'] = lines[4][:-1]
                app.config['MYSQL_DATABASE_PASSWORD'] = lines[5][:-1]
                app.config['MYSQL_DATABASE_DB'] = lines[6][:-1]
                app.config['MYSQL_DATABASE_HOST'] = lines[7][:-1]
        else:
                app.config['MYSQL_DATABASE_USER'] = lines[0][:-1]
                app.config['MYSQL_DATABASE_PASSWORD'] = lines[1][:-1]
                app.config['MYSQL_DATABASE_DB'] = lines[2][:-1]
                app.config['MYSQL_DATABASE_HOST'] = lines[3][:-1]

	#intialize app and create tables if doesn't exists
	mysql.init_app(app)
	conn = mysql.connect()
        cursor = conn.cursor()
        cursor.execute("CREATE TABLE IF NOT EXISTS user (`id` BIGINT NULL AUTO_INCREMENT,`name` VARCHAR(45) NULL,`color` VARCHAR(45) NULL,`ip` VARCHAR(45) NULL,PRIMARY KEY (`id`));")
        conn.commit()
	cursor.execute("CREATE TABLE IF NOT EXISTS room (   `id` BIGINT NULL AUTO_INCREMENT,   `name` VARCHAR(45) NULL,   PRIMARY KEY (`id`));")
        conn.commit()
        cursor.execute("CREATE TABLE IF NOT EXISTS room_user ( `id_room_user` INT NOT NULL AUTO_INCREMENT, `id_user` INT NOT NULL REFERENCES user(`id`), `id_room` INT NOT NULL REFERENCES room(`id`), PRIMARY KEY (`id_room_user`));")
        conn.commit()

	#Verify if users and rooms default are already saved
	cursor.execute("SELECT * FROM user")
    	data=cursor.fetchall()
	if len(data)==0:
		insert_user("moises", "#DAA520", "0.0.0.0", cursor, conn)	
		insert_user("glauco", "#4169E1", "0.0.0.0", cursor, conn)	
		insert_user("patricia", "#FFBBFF", "0.0.0.0", cursor, conn)	
	cursor.execute("SELECT * FROM room")
   	data=cursor.fetchall()
	if len(data)==0:
		cursor.execute("INSERT INTO room (name) VALUES('default')")
		conn.commit()
	conn.commit()
#if a backend charm is not connected with a charm MySQL, an error happens
except IOError:
	print "Not connected with a MySQL Charm"
@app.errorhandler(400)

#method that up a bad request
def bad_request(error):
    return make_response(jsonify({'error': 'Bad request'}), 400)

#method that returns an error message
@app.errorhandler(404)
def not_found(error):
    return make_response(jsonify({'error': 'Not found'}), 404)

#TODO: delete this tasks, are unecessary

tasks = [
    {
        'id': 1,
        'title': u'Buy groceries',
        'description': u'Milk, Cheese, Pizza, Fruit, Tylenol',
        'done': False
    },
    {
        'id': 2,
        'title': u'Learn Python',
        'description': u'Need to find a good Python tutorial on the web',
        'done': False
    }
]

#retrieve users that are saved in database
def get_users_from_database():
    #connects with mysql database
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute("SELECT * FROM user")
    data=cursor.fetchall()
    #Creates a list user to be returned
    users=[]
    #if select returns something, if is executed and get user values
    if len(data)!=0:
	for userData in data:
		user={}
		user['id']=userData[0]
                user['name']=userData[1]
		user['color']=userData[2]
		user['ip']=userData[3]
		users.append(user)
	#TODO: close mysql connection	
	return users

#retrieve users from database
def get_rooms_from_database():
    #Connects with mysql database
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute("SELECT * FROM room")
    data=cursor.fetchall()
    #Creates a room list to be returned
    rooms=[]
    #if select returns something, if is executed and get room values
    if len(data)!=0:
        for roomData in data:
                room={}
                room['id']=roomData[0]
                room['name']=roomData[1]
                room['messages']=[]
                room['users']=[]
                rooms.append(room)
	#TODO: close mysql connection
        return rooms

#save users and rooms retrieved from database 
users = get_users_from_database()
rooms = get_rooms_from_database()
#-----------------------------------------------------------------
#API for user
#-----------------------------------------------------------------

#method that turns user public in API
def make_public_user(user):
    new_user = {}
    print user
    for field in user:
        if field == 'id':
            new_user['uri'] = url_for('get_user', user_name=user['name'],
                                      _external=True)
        else:
            new_user[field] = user[field]
    return new_user

#return users via GET request
@app.route('/users', methods=['GET', 'OPTIONS'])
def get_users():
    return jsonify({'users': [make_public_user(user) for user in users]})

#return a specific user via GET
@app.route('/users/<user_name>', methods=['GET'])
def get_user(user_name):
    user = [user for user in users if user['name'] == user_name]
    if len(user) == 0:
          abort(404)
    return jsonify({'user': make_public_user(user[0])})

#Create or update a user in users via POST
@app.route('/users', methods=['POST'])
def create_user():
    if not request.json:
        abort(400)
    user = [user for user in users if user['name'] == request.json['name']]
    #UPDATES IP IF USER EXISTS
    if len(user) != 0:
     	user[0]['ip'] = request.json['ip']
	return jsonify({'user': make_public_user(user[0])})
    #IF NOT, CREATES OTHER
    user = {
        'id': users[-1]['id'] + 1,
        'name': request.json['name'],
        'color': request.json.get('color', ""),
        'ip': request.json['ip'],
	'room': request.json['room']
    }
    #saves user and room in database
    users.append(user)
    save_user(user)
    save_room(user)
    return jsonify({'user': make_public_user(user)}), 201


#saves a new user in database
#first, verify if it exists
#if not exists, create, else, not create
def save_user(user):
    conn=mysql.connect()
    cursor=conn.cursor()
    cursor.execute("SELECT * FROM user where name LIKE '"+user['name']+"'")
    response=cursor.fetchall()
    if len(response)==0:
    		conn = mysql.connect()
    		cursor = conn.cursor()
		insert_user(str(user['name']), str(user['color']), str(user['ip']), cursor, conn)
		return True
    return False

#saves a room in database
#same method of user: if exists, not create. Else, creates
def save_room(user):
	room=user['room']
	name_user=user['name']
	conn = mysql.connect()
    	cursor = conn.cursor()
	cursor.execute("SELECT * FROM room where name LIKE '"+room+"'")
	response=cursor.fetchall()
	#if room doesnt exist, create
	if len(response)==0:
		cursor=conn.cursor()
		cursor.execute("INSERT INTO room (name) VALUES ('"+room+"')")
		conn.commit()
	#get user id to save in table room_user that relates users that are in rooms
	cursor.execute("SELECT id FROM user WHERE name LIKE '"+name_user+"'")
	response_user=cursor.fetchall()
	if(len(response_user)!=0):
		id_user=response_user[0][0]
		cursor.execute("SELECT id FROM room WHERE name LIKE '"+room+"'")
		response_room=cursor.fetchall()
		id_room=response_room[0][0]
		cursor.execute("INSERT INTO room_user(id_user,id_room) VALUES ('"+str(id_user)+"', '"+str(id_room)+"')")
		conn.commit()
		return True
	return False

#-----------------------------------------------------------------------
#API for rooms
#-----------------------------------------------------------------------
def make_public_room(room):
    new_room = {}
    for field in room:
        if field == 'id':
            new_room['uri'] = url_for('get_room', room_name=room['name'],
                                      _external=True)
        else:
            new_room[field] = room[field]
    return new_room

#get rooms available
@app.route('/rooms', methods=['GET', 'OPTIONS'])
def get_rooms():
    return jsonify({'rooms': [make_public_room(room)for room in rooms]})

#get specific room
@app.route('/rooms/<room_name>', methods=['GET'])
def get_room(room_name):
    room = [room for room in rooms if room['name'] == room_name]
    if len(room) == 0:
          abort(404)
    return jsonify({'room': make_public_room(room[0])})

#creates or update room via POST method
@app.route('/rooms', methods=['POST'])
def create_room():
    if not request.json:
        abort(400)
    #update room 
    room = [room for room in rooms if room['name'] == request.json['name']]
    #if room exists, update room
    #this is made when new users connect with room
    if len(room) != 0:
        room[0]['name'] = request.json.get('name', room[0]['name'])
        room[0]['messages']+=request.json['messages']
	#verify if user there is in list users and put in final of list
	try:
		room[0]['users'].remove(request.json['users'][0])
		room[0]['users'].extend(request.json['users'])
	except ValueError:	
        	room[0]['users'].extend(request.json['users'])
        return jsonify({'room': make_public_room(room[0])})
    #if not create room
    room = {
        'id': rooms[-1]['id'] + 1,
        'name': request.json['name'],
        'messages': request.json['messages'],
	'users': request.json['users']
    }
    rooms.append(room)
    return jsonify({'room': make_public_room(room)}), 201

#method to insert new messages
@app.route('/rooms/<room_name>', methods=['POST', 'PUT'])
def update_room(room_name):
    if not request.json:
        abort(400)
    #update messages via PUT
    room = [room for room in rooms if room['name'] == room_name]
    if request.method == 'POST':
    	room[0]['messages'].append(request.json)
    elif request.method == 'PUT':
	room[0]['name'] = request.json.get('name', room[0]['name'])
    	room[0]['messages']=request.json.get('messages',room[0]['messages'])
    	room[0]['users']=request.json['room']['users']
    
    return jsonify({'room': make_public_room(room[0])}),201	

#---------------------------------------------------------------------------------------

#REST TO FRAMEWORK NOPAAS
@app.route('/state/restore', methods=['POST'])
def restore():
        global users
        users=request.json['base']['users']
        global rooms
        rooms=request.json['base']['rooms']
        return jsonify({'response': 'OK'}),200

@app.route('/state/base')
def base():
        base={
		'base':
			{
			'charm_name':CHARM_NAME,
			'charm_unit_name':CHARM_UNIT_NAME,
			'users':[],
			'rooms':[]
		}
	}

        base['base']['users']=users
        base['base']['rooms']=rooms
        return jsonify(base),200

@app.route('/state/basetime')
def basetime():
        x=1
        return None

@app.route('/state/diff')
def diff():
        x=1
        return None

@app.route('/state/difftime')
def difftime():
        x=1
        return None

@app.route('/state/period')
def period():
        x=1
        return None

#get the session information
#in backend case, the room information
@app.route('/state/session/<room_name>')
def get_session(room_name):
	return get_room(room_name)

if __name__ == '__main__':
  app.run(debug=True,host='0.0.0.0')
