#!flask/bin/python
from flask import Flask, session, redirect, url_for, render_template, request, jsonify
from forms import LoginForm
import os
import time
import requests
from websocket import create_connection
import urllib2
import json
#
users={}

app = Flask(__name__)
app.config['SECRET_KEY'] = 'gjr39dkjn344_!67#'
csrf_token=''
#get the log_backend to verify which server is the backend
#An actual problem is that only one server is recognizesd as backend
#When another server is created in scaling process, the script in hooks folder delete older file and creates a new with new IP address of last server
#TODO: make file recognize two backends in scaling (hooks folder)
filename=open('../log_backend.txt','r')
lines=filename.readlines()
ip_server=lines[0][:-1]

#creates connection with websocket server
#this websocket_server.py is responsible for notify events as new messages or new users joined in the room
ws=create_connection("ws://localhost:5050/")

#login page
@app.route('/', methods=['GET', 'POST'])
def index():
    """"Login form to enter a room."""
    form = LoginForm()
    if form.validate_on_submit():
	#when user and room informations are send, system store a session and redirect to chat method
	#this session is deprecated and will be deleted soon
        session['name'] = form.name.data
        session['room'] = form.room.data
	return redirect(url_for('.chat'))
    elif request.method == 'GET':
	#get csrf_token from chat_backend.py running in backend
	#this will be modified because session will be stored differently
	global csrf_token
	csrf_token=requests.get('http://'+ip_server+':5050/').content
    return render_template('index.html', form=form)

#chat room
@app.route('/chat')
def chat():
    """Chat room. The user's name and room must be stored in
    the session."""
    #get session user and sends name and room to chat_backend.py in backend 
    name = session.get('name', '')
    room = session.get('room', '')
    global csrf_token
    global ip_server
    data = {
        'name': name,
        'room': room,
	'csrf_token':csrf_token,
}

    req = urllib2.Request('http://'+ip_server+':5050/chat')
    req.add_header('Content-Type', 'application/json')
    #response is the room created or updated
    #room information is used to verify the last user that was connected
    #and informs to backend.py that user entered the room
    response = urllib2.urlopen(req, json.dumps(data))
    room_json=json.loads(response.read())
    room_name=room_json['room']['name']
    new_user=room_json['room']['users'][-1]

    #get user that entered in room information
    user = json.loads(requests.get('http://'+ip_server+':5000/users/'+new_user).content)
	
    #get color from user
    color=user['user']['color']
    user_name=user['user']['name']
    #if users, dict of users that are connected to frontend, not have the new user, it is insert
    #the key is your name and the value, and data a set of rooms and the color desired
    if (user_name not in users.keys()):
	users[user_name]=(set([room_name]),color)
    #if exists, only adds the new room in set of user's rooms
    else:
    	users[user_name][0].add(room_name)

    #websocket send message that a new user entered in room
    ws.send(json.dumps({'room':room_name,'user':new_user,'action':'new_user'}))

    if name == '' or room == '':
        return redirect(url_for('.index'))
    #render page with necessary informations as ip_backend, room, name of user and color 
    return render_template('chat.html', ip=ip_server, room=room, name=name, color=color)

#send a message to backend 
#this message is sent by client in browser
#this method control how the message is send to backend
@app.route('/sendMessage', methods=['POST'])
def send_message():
	#retrieve message, room and user that sent
	message=request.json['message']
	room=request.json['room']
	user=request.json['user']
	#creates the message, set:
	#-where will be sent (room of backend)
	#-type of file (json)
	#-message to room (user+message) 
	req = urllib2.Request('http://'+ip_server+':5000/rooms/'+room)
    	req.add_header('Content-Type', 'application/json')
	complete_message={'user':user,'message':message}
	#make a post in desired room of message was sent
	#resposta_json receive the room with message saved in backend
	#and send via websocket to clients
    	response = urllib2.urlopen(req, json.dumps(complete_message))
	resposta_json=json.loads(response.read())
	last_message=resposta_json['room']['messages'][-1]
	ws.send(json.dumps({'room':resposta_json['room']['name'],'user':last_message['user'],'message':last_message['message']}))
	#???
	#Will see in soon	
	last_message['user']=""
	last_message['message']="" 
	return jsonify(last_message),200

#method that logout user when it clicks in "Leave Room" on chat page
@app.route('/logout', methods=['GET'])
def logout():
	room=request.args['amp;room']
	user=request.args['name']
	#gets the room that user will logout
	#remove user
	#make a post in backend without the user
	get_room=json.loads(requests.get('http://'+ip_server+':5000/rooms/'+room).content)
	get_room['room']['users'].remove(user)
	req = requests.put('http://'+ip_server+':5000/rooms/'+room, data=json.dumps(get_room), headers={'Content-type':'application/json'})
	#remove room of user in dict users	
	users[user][0].remove(room)
	#warn via websocket that user left room
	ws.send(json.dumps({'room':room,'user':user,'action':'left_user'}))
	#redirect from index page
	return redirect(url_for('.index'))

#gets session information from frontend
@app.route('/state/session/<user_name>')
def get_session(user_name):
	try:
		data={
			"user": user_name,
			"rooms": list(users[user_name][0]),
			"color": users[user_name][1]
		}
	except KeyError:
		return jsonify({'error': 'Not Found'}), 400
	return jsonify(data), 200

if __name__ == '__main__':
  app.run(debug=True,host='0.0.0.0', port=5000)

