#!flask/bin/python
import six
from flask import Flask, jsonify, abort, request, make_response, url_for, render_template, json
from flask.ext.httpauth import HTTPBasicAuth
from flask.ext.mysql import MySQL
from werkzeug import generate_password_hash, check_password_hash

app = Flask(__name__, static_url_path="")
auth = HTTPBasicAuth()

try:
	mysql = MySQL()
	
	info=open('../log_mysql.txt', 'r')
	lines=info.readlines()
	# MySQL configurations
	app.config['MYSQL_DATABASE_USER'] = lines[4]
	app.config['MYSQL_DATABASE_PASSWORD'] = lines[5]
	app.config['MYSQL_DATABASE_DB'] = lines[6]
	app.config['MYSQL_DATABASE_HOST'] = lines[7]
	mysql.init_app(app)
	
except IOError:
	print "Not connected with a MySQL Charm"
@app.errorhandler(400)
def bad_request(error):
    return make_response(jsonify({'error': 'Bad request'}), 400)


@app.errorhandler(404)
def not_found(error):
    return make_response(jsonify({'error': 'Not found'}), 404)


tasks = [
    {
        'id': 1,
        'title': u'Buy groceries',
        'description': u'Milk, Cheese, Pizza, Fruit, Tylenol',
        'done': False
    },
    {
        'id': 2,
        'title': u'Learn Python',
        'description': u'Need to find a good Python tutorial on the web',
        'done': False
    }
]


def make_public_task(task):
    new_task = {}
    for field in task:
        if field == 'id':
            new_task['uri'] = url_for('get_task', task_id=task['id'],
                                      _external=True)
        else:
            new_task[field] = task[field]
    return new_task


@app.route('/todo/api/v1.0/tasks', methods=['GET'])
def get_tasks():
    return jsonify({'tasks': [make_public_task(task) for task in tasks]})


@app.route('/todo/api/v1.0/tasks/<int:task_id>', methods=['GET'])
def get_task(task_id):
    task = [task for task in tasks if task['id'] == task_id]
    if len(task) == 0:
          abort(404)
    return jsonify({'task': make_public_task(task[0])})


@app.route('/todo/api/v1.0/tasks', methods=['POST'])
def create_task():
    if not request.json or 'title' not in request.json:
        abort(400)
    task = {
        'id': tasks[-1]['id'] + 1,
        'title': request.json['title'],
        'description': request.json.get('description', ""),
        'done': False
    }
    tasks.append(task)
    return jsonify({'task': make_public_task(task)}), 201


@app.route('/todo/api/v1.0/tasks/<int:task_id>', methods=['PUT'])
def update_task(task_id):
    task = [task for task in tasks if task['id'] == task_id]
    if len(task) == 0:
        abort(404)
    if not request.json:
        abort(400)
    if 'title' in request.json and \
            not isinstance(request.json['title'], six.string_types):
        abort(400)
    if 'description' in request.json and \
            not isinstance(request.json['description'], six.string_types):
        abort(400)
    if 'done' in request.json and type(request.json['done']) is not bool:
     	abort(400)
    task[0]['title'] = request.json.get('title', task[0]['title'])
    task[0]['description'] = request.json.get('description',
                                              task[0]['description'])
    task[0]['done'] = request.json.get('done', task[0]['done'])
    return jsonify({'task': make_public_task(task[0])})


@app.route('/todo/api/v1.0/tasks/<int:task_id>', methods=['DELETE'])
def delete_task(task_id):
    task = [task for task in tasks if task['id'] == task_id]
    if len(task) == 0:
        abort(404)
    tasks.remove(task[0])
    return jsonify({'result': True})

@app.route('/showSignUp')
def showSignUp():
    return render_template('signup.html')

@app.route('/signUp',methods=['POST'])
def signUp():
    _name = request.form['inputName']
    _email = request.form['inputEmail']
    _password = request.form['inputPassword']
    #print _name, _email
    if _name and _email and _password:
        conn = mysql.connect()
        cursor = conn.cursor()

        #_hashed_password = generate_password_hash(_password)
        cursor.callproc('sp.createUser',(_name,_email,_password))
	data = cursor.fetchall()
        if len(data) is 0:
                conn.commit()
                return json.dumps({'message':'User created sucessfully!'})
        else:
                return json.dumps({'error':str(data[0])})
    else:
        print "false"
        return json.dumps({'html':'<span>Enter the required fields</span>'})

if __name__ == '__main__':
  app.run(debug=True,host='0.0.0.0')

